==> gradle-8.14.4-20260127040244+0000-src.zip: gradle-8.14.4-20260127040244+0000/platforms/software/dependency-management/src/main/java/org/gradle/api/internal/artifacts/ivyservice/dependencysubstitution/DependencySubstitutionApplicator.java <==
/*
 * Copyright 2017 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.gradle.api.internal.artifacts.ivyservice.dependencysubstitution;

import com.google.common.collect.ImmutableList;
import org.gradle.api.internal.artifacts.ivyservice.resolveengine.graph.builder.DependencyState;
import org.gradle.internal.component.model.DependencyMetadata;

/**
 * A dependency substitution applicator is responsible for applying substitution rules to dependency metadata.
 * Substitution result may either be the same module (no substitution), a different module (target of the substitution
 * is going to be different) or a failure.
 */
public interface DependencySubstitutionApplicator {

    /**
     * A substitution applicator that does not perform any substitutions.
     */
    DependencySubstitutionApplicator NO_OP = metadata -> new DependencyState(metadata, metadata.getSelector(), ImmutableList.of(), null);

    /**
     * Execute any dependency substitution rules that apply to the given dependency metadata.
     *
     * @return a dependency state representing the result of applying any substitution rules.
     */
    DependencyState applySubstitutions(DependencyMetadata metadata);

}

==> gradle-9.4.0-20260209035730+0000-src.zip: gradle-9.4.0-20260209035730+0000/platforms/software/dependency-management/src/main/java/org/gradle/api/internal/artifacts/ivyservice/dependencysubstitution/DependencySubstitutionApplicator.java <==
/*
 * Copyright 2017 the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.gradle.api.internal.artifacts.ivyservice.dependencysubstitution;

import com.google.common.collect.ImmutableList;
import org.gradle.api.artifacts.component.ComponentSelector;
import org.gradle.internal.Try;
import org.gradle.internal.component.model.IvyArtifactName;
import org.jspecify.annotations.NullMarked;

/**
 * A dependency substitution applicator is responsible for applying substitution rules to dependency metadata.
 * Substitution result may either be the same module (no substitution), a different module (target of the substitution
 * is going to be different) or a failure.
 */
@NullMarked
public interface DependencySubstitutionApplicator {

    /**
     * A substitution applicator that does not perform any substitutions.
     */
    DependencySubstitutionApplicator NO_OP = (selector, artifacts) -> Try.successful(DefaultDependencySubstitutionApplicator.DefaultSubstitutionResult.NO_OP);

    /**
     * Execute any dependency substitution rules that apply to the given dependency sector and artifacts.
     *
     * @return the result of applying any substitution rules.
     */
    Try<SubstitutionResult> applySubstitutions(ComponentSelector selector, ImmutableList<IvyArtifactName> artifacts);

}
